@@ -10,6 +10,7 @@
 #include "google/protobuf/arena.h"
 #include "google/protobuf/test_util.h"
 #include "google/protobuf/unittest.pb.h"
+#include "google/protobuf/unittest_import.pb.h"
 
 namespace google {
 namespace protobuf {
@@ -18,27 +19,27 @@ namespace cpp {
 namespace {
 
 TEST(CopyMessageTest, CopyConstructor) {
-  protobuf_unittest::TestAllTypes message1;
+  proto2_unittest::TestAllTypes message1;
   TestUtil::SetAllFields(&message1);
-  protobuf_unittest::TestAllTypes message2(message1);
+  proto2_unittest::TestAllTypes message2(message1);
   TestUtil::ExpectAllFieldsSet(message2);
 }
 
 TEST(CopyMessageTest, ArenaEnabledCopyConstructorNull) {
-  protobuf_unittest::TestAllTypes message1;
+  proto2_unittest::TestAllTypes message1;
   TestUtil::SetAllFields(&message1);
-  protobuf_unittest::TestAllTypes* message2 =
-      Arena::Create<protobuf_unittest::TestAllTypes>(nullptr, message1);
+  proto2_unittest::TestAllTypes* message2 =
+      Arena::Create<proto2_unittest::TestAllTypes>(nullptr, message1);
   TestUtil::ExpectAllFieldsSet(*message2);
   delete message2;
 }
 
 TEST(CopyMessageTest, ArenaEnabledCopyConstructor) {
-  protobuf_unittest::TestAllTypes message1;
+  proto2_unittest::TestAllTypes message1;
   TestUtil::SetAllFields(&message1);
   Arena arena;
-  protobuf_unittest::TestAllTypes* message2 =
-      Arena::Create<protobuf_unittest::TestAllTypes>(&arena, message1);
+  proto2_unittest::TestAllTypes* message2 =
+      Arena::Create<proto2_unittest::TestAllTypes>(&arena, message1);
   TestUtil::ExpectAllFieldsSet(*message2);
 }
 
@@ -46,14 +47,14 @@ TEST(CopyMessageTest, ArenaEnabledCopyConstructorArenaLeakTest) {
   // Set possible leaking field types for TestAllTypes with values
   // guaranteed to not be inlined string or Cord values.
   // TestAllTypes has unconditional ArenaDtor registration.
-  protobuf_unittest::TestAllTypes message1;
+  proto2_unittest::TestAllTypes message1;
 
   message1.set_optional_string(std::string(1000, 'a'));
   message1.add_repeated_string(std::string(1000, 'd'));
 
   Arena arena;
-  protobuf_unittest::TestAllTypes* message2 =
-      Arena::Create<protobuf_unittest::TestAllTypes>(&arena, message1);
+  proto2_unittest::TestAllTypes* message2 =
+      Arena::Create<proto2_unittest::TestAllTypes>(&arena, message1);
 
   EXPECT_EQ(message2->optional_string(), message1.optional_string());
   EXPECT_EQ(message2->repeated_string(0), message1.repeated_string(0));
